/* qSSHscan.java */

/*****************************************************************/
/*  I made this during the boring parts of java class.  I hope   */
/* you will find it as useful as I do.  It can run on linux or   */
/* windows, is not logged by default opensshd, and is relatively */
/* fast.  You can tweak its speed by changing the MAX_THREADS    */
/* and TIMEOUT variables.  You can also set PRUNE to true if you */
/* want to print alls hosts without open and connect()able SSH.  */
/*                                                               */
/* -- j0ker [j0ker@daforest.org]                                 */
/* july 2003                                                     */
/* http://olives.ath.cx/~j0ker/                                  */

/* c:> javac qSSHscan.java                                       */
/* c:> java -cp . qSSHscan <input_file> <port> <prune?>          */
/* c:> java -cp . qSSHscan list.in 22 yes > list.out             */
/* c:> java -cp . qSSHscan list.in 22 no > list.out              */
/* just be sure you have java 1.4 or higher...                   */

import java.net.*;
import java.io.*;

public class qSSHscan
{
  /* k, heres the dilly-o */
  public static void main(String args[]) throws IOException
  {
    /* variables */
    String HOST = "", wrk = "", VERSION = "1.1", FILENAME = "";
    BufferedReader read_input = null;
    boolean PRUNE = true; /* set PRUNE to false to print hosts without open and connect()able SSH */
    int PORT = 22, MAX_THREADS = 28, TIMEOUT = 1700, j = 0;
    /* default MAX_THREADS of 28 threads seems like plenty, but add or subtract as you need */
    /* default TIMEOUT of 1.7 seconds, try .5 seconds or even less for fast networks, 2 to 2.5 seems to avoid false negatives */

    /* greetings */
    System.out.println("quietSSHscan (qSSHscan) v" + VERSION + " by j0ker@daforest.org");
	  System.out.println("code is nothing special, but written in java for portability");
	  System.out.println("scans will NOT be logged by opensshd default settings");
	  System.out.println("change MAX_THREADS and TIMEOUT vars in code to tweak speed");

    /* check for all args and set variables */
    if (args.length < 3)
    {
			System.out.println("-> usage: java -cp . qSSHscan <input_file> <port> <prune?>");
			System.out.println("-> example: java -cp . qSSHscan file.in 22 no");
			System.out.println("->          reads ips/hosts from file.in, scans p22, prints all results");
			System.out.println("-> example: java -cp . qSSHscan file.in 22 yes");
			System.out.println("->          same as above, but prints only hosts with p22 SSH open");
			System.out.println("-> use piping to save results, consult README or source comments");
			System.exit(0);
		}
    FILENAME = args[0];
    PORT = Integer.parseInt(args[1]);
    if(args[2].equalsIgnoreCase("no")) { PRUNE = false; }

    /* state PRUNE status */
    System.out.print("PRUNE is: ");
		if(PRUNE) System.out.print("on, only printing hosts with SSH\n");
	  else System.out.print("off, printing all hosts\n");

    /* start reading from input file, input can be either ips or hostnames */
    read_input = new BufferedReader(new FileReader(new File(FILENAME)));

	  /* threads are so easy in java, it feels like cheating */
	  ThreadGroup scanThreads = new ThreadGroup("scanThreads");
	  HOST = read_input.readLine();
    while(HOST != null)
    {
			if(scanThreads.activeCount() < MAX_THREADS)
			{
				Thread child = new Thread(scanThreads, new scanThread(HOST, PORT, PRUNE, TIMEOUT));
		    child.start();
		    HOST = read_input.readLine();
		  }
	  }

    /* cleanup and exit */
    if(read_input != null) read_input.close();
    //System.exit(0);
  }
}

/* the thread class */
class scanThread extends Thread
{
	private String HOST;
  private int PORT, TIMEOUT;
  private boolean PRUNE;

  /* set up variables */
	public scanThread(String wrk1, int wrk2, boolean wrk3, int wrk4)
	{
    HOST = wrk1;
    PORT = wrk2;
    PRUNE = wrk3;
    TIMEOUT = wrk4;
	}

	/* actually do something */
	public void run()
	{
    String mock_version_string = "SSH-2.0-OpenSSH_3.5p1";
	  PrintWriter write_socket = null;
    BufferedReader read_socket = null;
		SocketAddress sockaddr = null;
    Socket sock = new Socket();

		try
		{
			/* make a socket with TIMEOUT timeout */
			sockaddr = new InetSocketAddress(HOST, PORT);
			sock.connect(sockaddr, TIMEOUT);
			sock.setSoTimeout(TIMEOUT);

		  /* need this to be literate on the socket */
		  read_socket = new BufferedReader(new InputStreamReader(sock.getInputStream()));
		  write_socket = new PrintWriter(sock.getOutputStream());

		  /* get, print, and save the version info */
			System.out.println("-> " + HOST + ":" + PORT + " -[ " + read_socket.readLine() + " ]");

			/* "trick" opensshd into not printing a vile "Did not recieve identification string from xxx" log message */
			write_socket.println(mock_version_string);
		  write_socket.flush();
		}
		catch(IOException e)
		{
			if(!PRUNE)
		  System.out.println("-> " + HOST + ":" + PORT + " -[ " + e + " ]");
	  }

    try
    {
			if(sock != null) sock.close();
	    if(read_socket != null) read_socket.close();
		  if(write_socket != null) write_socket.close();
	  } catch (IOException e) {}
	}
}
